@@ -56,7 +56,7 @@ KImGalleryPlugin::KImGalleryPlugin(QObject *parent, const QVariantList &)
     QAction *a = actionCollection()->addAction(QStringLiteral("create_img_gallery"));
     a->setText(i18n("&Create Image Gallery..."));
     a->setIcon(QIcon::fromTheme(QStringLiteral("imagegallery")));
-    actionCollection()->setDefaultShortcut(a, QKeySequence(Qt::CTRL + Qt::Key_I));
+    actionCollection()->setDefaultShortcut(a, QKeySequence(Qt::CTRL | Qt::Key_I));
     connect(a, &QAction::triggered, this, &KImGalleryPlugin::slotExecute);
 }
 
