@@ -18,6 +18,24 @@
  *
  */
 
+/*
+ * The code for the function slotPrintPreview was adapted from the PrintMe Qt example
+ * and is distributed under the BSD license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are
+ * met:
+ *   * Redistributions of source code must retain the above copyright
+ *     notice, this list of conditions and the following disclaimer.
+ *   * Redistributions in binary form must reproduce the above copyright
+ *     notice, this list of conditions and the following disclaimer in
+ *     the documentation and/or other materials provided with the
+ *     distribution.
+ *   * Neither the name of The Qt Company Ltd nor the names of its
+ *     contributors may be used to endorse or promote products derived
+ *     from this software without specific prior written permission.
+ */
+
 #include "webenginepart_ext.h"
 #include <QtWebEngine/QtWebEngineVersion>
 
@@ -42,6 +60,7 @@
 #include <sonnet/backgroundchecker.h>
 #include <KIO/JobUiDelegate>
 #include <KIO/OpenUrlJob>
+#include <KParts/BrowserRun>
 
 #include <QBuffer>
 #include <QVariant>
@@ -53,6 +72,7 @@
 #include <QPrintPreviewDialog>
 #include <QWebEngineHistory>
 #include <QMimeData>
+#include <QPrinterInfo>
 #define QL1S(x)     QLatin1String(x)
 #define QL1C(x)     QLatin1Char(x)
 
@@ -117,20 +137,18 @@ WebEngineView* WebEngineBrowserExtension::view()
 
 int WebEngineBrowserExtension::xOffset()
 {
-#if QTWEBENGINE_VERSION >= QT_VERSION_CHECK(5, 7, 0)
-    if (view())
+    if (view()) {
         return view()->page()->scrollPosition().x();
-#endif
+    }
 
     return KParts::BrowserExtension::xOffset();
 }
 
 int WebEngineBrowserExtension::yOffset()
 {
-#if QTWEBENGINE_VERSION >= QT_VERSION_CHECK(5, 7, 0)
-   if (view())
+   if (view()) {
         return view()->page()->scrollPosition().y();
-#endif
+   }
 
     return KParts::BrowserExtension::yOffset();
 }
@@ -182,7 +200,7 @@ void WebEngineBrowserExtension::restoreState(QDataStream &stream)
                         if (QCoreApplication::applicationName() == QLatin1String("konqueror")) {
                             history->clear();
                         }
-                        //kDebug() << "Restoring URL:" << currentItem.url();
+                        //qCDebug(WEBENGINEPART_LOG) << "Restoring URL:" << currentItem.url();
                         m_part->setProperty("NoEmitOpenUrlNotification", true);
                         history->goToItem(currentItem);
                     }
@@ -190,10 +208,10 @@ void WebEngineBrowserExtension::restoreState(QDataStream &stream)
             }
             success = (history->count() > 0);
         } else {        // Handle navigation: back and forward button navigation.
-            //kDebug() << "history count:" << history->count() << "request index:" << historyItemIndex;
+            //qCDebug(WEBENGINEPART_LOG) << "history count:" << history->count() << "request index:" << historyItemIndex;
             if (history->count() > historyItemIndex && historyItemIndex > -1) {
                 QWebEngineHistoryItem item (history->itemAt(historyItemIndex));
-                //kDebug() << "URL:" << u << "Item URL:" << item.url();
+                //qCDebug(WEBENGINEPART_LOG) << "URL:" << u << "Item URL:" << item.url();
                 if (u == item.url()) {
                     if (item.isValid() && (xOfs != -1 || yOfs != -1)) {
                         const QPoint scrollPos (xOfs, yOfs);
@@ -250,7 +268,6 @@ void WebEngineBrowserExtension::slotSaveFrame()
 
 void WebEngineBrowserExtension::print()
 {
-#if QTWEBENGINE_VERSION >= QT_VERSION_CHECK(5, 8, 0)
     if (view()) {
         mCurrentPrinter = new QPrinter();
         QPointer<QPrintDialog> dialog = new QPrintDialog(mCurrentPrinter, nullptr);
@@ -263,7 +280,6 @@ void WebEngineBrowserExtension::print()
         delete dialog;
         view()->page()->print(mCurrentPrinter, invoke(this, &WebEngineBrowserExtension::slotHandlePagePrinted));
     }
-#endif
 }
 
 void WebEngineBrowserExtension::slotHandlePagePrinted(bool result)
@@ -400,9 +416,9 @@ void WebEngineBrowserExtension::slotSaveImageAs()
 
 void WebEngineBrowserExtension::slotSendImage()
 {
-#if QTWEBENGINE_VERSION >= QT_VERSION_CHECK(5, 7, 0)
-    if (!view())
+    if (!view()) {
         return;
+    }
 
     QStringList urls;
     urls.append(view()->contextMenuResult().mediaUrl().path());
@@ -411,14 +427,13 @@ void WebEngineBrowserExtension::slotSendImage()
                                   QString(), //body
                                   QString(),
                                   urls); // attachments
-#endif
 }
 
 void WebEngineBrowserExtension::slotCopyImageURL()
 {
-#if QTWEBENGINE_VERSION >= QT_VERSION_CHECK(5, 7, 0)
-    if (!view())
+    if (!view()) {
         return;
+    }
 
     QUrl safeURL = view()->contextMenuResult().mediaUrl();
     safeURL.setPassword(QString());
@@ -433,15 +448,14 @@ void WebEngineBrowserExtension::slotCopyImageURL()
     mimeData = new QMimeData;
     mimeData->setUrls(safeURLList);
     QApplication::clipboard()->setMimeData(mimeData, QClipboard::Selection);
-#endif
 }
 
 
 void WebEngineBrowserExtension::slotCopyImage()
 {
-#if QTWEBENGINE_VERSION >= QT_VERSION_CHECK(5, 7, 0)
-    if (!view())
+    if (!view()) {
         return;
+    }
 
     QUrl safeURL; //(view()->contextMenuResult().imageUrl());
     safeURL.setPassword(QString());
@@ -459,22 +473,20 @@ void WebEngineBrowserExtension::slotCopyImage()
 //    mimeData->setImageData(view()->contextMenuResult().pixmap());
     mimeData->setUrls(safeURLList);
     QApplication::clipboard()->setMimeData(mimeData, QClipboard::Selection);
-#endif
 }
 
 void WebEngineBrowserExtension::slotViewImage()
 {
-#if QTWEBENGINE_VERSION >= QT_VERSION_CHECK(5, 7, 0)
-    if (view())
+    if (view()) {
         emit createNewWindow(view()->contextMenuResult().mediaUrl());
-#endif
+    }
 }
 
 void WebEngineBrowserExtension::slotBlockImage()
 {
-#if QTWEBENGINE_VERSION >= QT_VERSION_CHECK(5, 7, 0)
-    if (!view())
+    if (!view()) {
         return;
+    }
 
     bool ok = false;
     const QString url = QInputDialog::getText(view(), i18n("Add URL to Filter"),
@@ -485,7 +497,6 @@ void WebEngineBrowserExtension::slotBlockImage()
         WebEngineSettings::self()->addAdFilter(url);
         reparseConfiguration();
     }
-#endif
 }
 
 void WebEngineBrowserExtension::slotBlockHost()
@@ -507,31 +518,32 @@ void WebEngineBrowserExtension::slotCopyLinkURL()
 
 void WebEngineBrowserExtension::slotCopyLinkText()
 {
-#if QTWEBENGINE_VERSION >= QT_VERSION_CHECK(5, 7, 0)
     if (view()) {
         QMimeData* data = new QMimeData;
         data->setText(view()->contextMenuResult().linkText());
         QApplication::clipboard()->setMimeData(data, QClipboard::Clipboard);
     }
-#endif
 }
 
 void WebEngineBrowserExtension::slotCopyEmailAddress()
 {
-#if QTWEBENGINE_VERSION >= QT_VERSION_CHECK(5, 7, 0)
     if (view()) {
         QMimeData* data = new QMimeData;
         const QUrl url(view()->contextMenuResult().linkUrl());
         data->setText(url.path());
         QApplication::clipboard()->setMimeData(data, QClipboard::Clipboard);
     }
-#endif
 }
 
-void WebEngineBrowserExtension::slotSaveLinkAs()
+void WebEngineBrowserExtension::slotSaveLinkAs(const QUrl &url)
 {
-    if (view())
-        view()->triggerPageAction(QWebEnginePage::DownloadLinkToDisk);
+    if (view()) {
+        if (!url.isEmpty()) {
+            KParts::BrowserRun::saveUrl(url, url.path(), view(), KParts::OpenUrlArguments());
+        } else {
+            view()->triggerPageAction(QWebEnginePage::DownloadLinkToDisk);
+        }
+    }
 }
 
 void WebEngineBrowserExtension::slotViewDocumentSource()
@@ -561,7 +573,6 @@ void WebEngineBrowserExtension::slotViewDocumentSource()
     }
 }
 
-#if QTWEBENGINE_VERSION >= QT_VERSION_CHECK(5, 7, 0)
 static bool isMultimediaElement(QWebEngineContextMenuData::MediaType mediaType)
 {
     switch(mediaType)
@@ -573,58 +584,53 @@ static bool isMultimediaElement(QWebEngineContextMenuData::MediaType mediaType)
             return false;
     }
 }
-#endif
 
 void WebEngineBrowserExtension::slotLoopMedia()
 {
-#if QTWEBENGINE_VERSION >= QT_VERSION_CHECK(5, 7, 0)
-    if (!view())
+    if (!view()) {
         return;
+    }
 
     QWebEngineContextMenuData data =  view()->contextMenuResult();
     if (!isMultimediaElement( data.mediaType()))
         return;
     view()->page()->triggerAction(QWebEnginePage::ToggleMediaLoop);
-#endif
 }
 
 void WebEngineBrowserExtension::slotMuteMedia()
 {
-#if QTWEBENGINE_VERSION >= QT_VERSION_CHECK(5, 7, 0)
-    if (!view())
+    if (!view()) {
         return;
+    }
 
     QWebEngineContextMenuData data =  view()->contextMenuResult();
     if (!isMultimediaElement( data.mediaType()))
         return;
     view()->page()->triggerAction(QWebEnginePage::ToggleMediaMute);
-#endif
 }
 
 void WebEngineBrowserExtension::slotPlayMedia()
 {
-#if QTWEBENGINE_VERSION >= QT_VERSION_CHECK(5, 7, 0)
-    if (!view())
+    if (!view()) {
         return;
+    }
 
     QWebEngineContextMenuData data =  view()->contextMenuResult();
     if (!isMultimediaElement( data.mediaType()))
         return;
     view()->page()->triggerAction(QWebEnginePage::ToggleMediaPlayPause);
-#endif
 }
 
 void WebEngineBrowserExtension::slotShowMediaControls()
 {
-#if QTWEBENGINE_VERSION >= QT_VERSION_CHECK(5, 7, 0)
-    if (!view())
+    if (!view()) {
         return;
+    }
 
     QWebEngineContextMenuData data =  view()->contextMenuResult();
     if (!isMultimediaElement( data.mediaType()))
         return;
     view()->page()->triggerAction(QWebEnginePage::ToggleMediaControls);
-#endif
 }
 
 #if 0
@@ -644,22 +650,21 @@ static QUrl mediaUrlFrom(QWebElement& element)
 
 void WebEngineBrowserExtension::slotSaveMedia()
 {
-#if QTWEBENGINE_VERSION >= QT_VERSION_CHECK(5, 7, 0)
-    if (!view())
+    if (!view()) {
         return;
+    }
 
     QWebEngineContextMenuData data =  view()->contextMenuResult();
     if (!isMultimediaElement( data.mediaType()))
         return;
     emit saveUrl(data.mediaUrl());
-#endif
 }
 
 void WebEngineBrowserExtension::slotCopyMedia()
 {
-#if QTWEBENGINE_VERSION >= QT_VERSION_CHECK(5, 7, 0)
-    if (!view())
+    if (!view()) {
         return;
+    }
     QWebEngineContextMenuData data =  view()->contextMenuResult();
     if (!isMultimediaElement( data.mediaType()))
         return;
@@ -680,7 +685,6 @@ void WebEngineBrowserExtension::slotCopyMedia()
     mimeData = new QMimeData;
     mimeData->setUrls(safeURLList);
     QApplication::clipboard()->setMimeData(mimeData, QClipboard::Selection);
-#endif
 }
 
 void WebEngineBrowserExtension::slotTextDirectionChanged()
@@ -708,8 +712,8 @@ void WebEngineBrowserExtension::slotCheckSpelling()
             backgroundSpellCheck->setParent(spellDialog);
             spellDialog->setAttribute(Qt::WA_DeleteOnClose, true);
             spellDialog->showSpellCheckCompletionMessage(true);
-            connect(spellDialog, SIGNAL(replace(QString,int,QString)), this, SLOT(spellCheckerCorrected(QString,int,QString)));
-            connect(spellDialog, SIGNAL(misspelling(QString,int)), this, SLOT(spellCheckerMisspelling(QString,int)));
+            connect(spellDialog, &Sonnet::Dialog::replace, this, &WebEngineBrowserExtension::spellCheckerCorrected);
+            connect(spellDialog, &Sonnet::Dialog::misspelling, this, &WebEngineBrowserExtension::spellCheckerMisspelling);
             spellDialog->setBuffer(text);
             spellDialog->show();
         }
@@ -726,16 +730,16 @@ void WebEngineBrowserExtension::slotSpellCheckSelection()
                 const int pos = values.indexOf(' ');
                 m_spellTextSelectionStart = qMax(0, values.leftRef(pos).toInt());
                 m_spellTextSelectionEnd = qMax(0, values.midRef(pos + 1).toInt());
-                // kDebug() << "selection start:" << m_spellTextSelectionStart << "end:" << m_spellTextSelectionEnd;
+                // qCDebug(WEBENGINEPART_LOG) << "selection start:" << m_spellTextSelectionStart << "end:" << m_spellTextSelectionEnd;
 
                 Sonnet::BackgroundChecker *backgroundSpellCheck = new Sonnet::BackgroundChecker;
                 Sonnet::Dialog* spellDialog = new Sonnet::Dialog(backgroundSpellCheck, view());
                 backgroundSpellCheck->setParent(spellDialog);
                 spellDialog->setAttribute(Qt::WA_DeleteOnClose, true);
                 spellDialog->showSpellCheckCompletionMessage(true);
-                connect(spellDialog, SIGNAL(replace(QString,int,QString)), this, SLOT(spellCheckerCorrected(QString,int,QString)));
-                connect(spellDialog, SIGNAL(misspelling(QString,int)), this, SLOT(spellCheckerMisspelling(QString,int)));
-                connect(spellDialog, SIGNAL(done(QString)), this, SLOT(slotSpellCheckDone(QString)));
+                connect(spellDialog, &Sonnet::Dialog::replace, this, &WebEngineBrowserExtension::spellCheckerCorrected);
+                connect(spellDialog, &Sonnet::Dialog::misspelling, this, &WebEngineBrowserExtension::spellCheckerMisspelling);
+                connect(spellDialog, &Sonnet::Dialog::spellCheckDone, this, &WebEngineBrowserExtension::slotSpellCheckDone);
                 spellDialog->setBuffer(text.mid(m_spellTextSelectionStart, (m_spellTextSelectionEnd - m_spellTextSelectionStart)));
                 spellDialog->show();
             });
@@ -759,13 +763,13 @@ void WebEngineBrowserExtension::spellCheckerCorrected(const QString& original, i
     script += QString::number(index + original.length());
     script += QL1S(")");
 
-    //kDebug() << "**** script:" << script;
+    //qCDebug(WEBENGINEPART_LOG) << "**** script:" << script;
     view()->page()->runJavaScript(script);
 }
 
 void WebEngineBrowserExtension::spellCheckerMisspelling(const QString& text, int pos)
 {
-    // kDebug() << text << pos;
+    // qCDebug(WEBENGINEPART_LOG) << text << pos;
     QString selectionScript (QL1S("this.setSelectionRange("));
     selectionScript += QString::number(pos + m_spellTextSelectionStart);
     selectionScript += QL1C(',');
@@ -793,7 +797,7 @@ void WebEngineBrowserExtension::saveHistory()
     QWebEngineHistory* history = (view() ? view()->history() : nullptr);
 
     if (history && history->count() > 0) {
-        //kDebug() << "Current history: index=" << history->currentItemIndex() << "url=" << history->currentItem().url();
+        //qCDebug(WEBENGINEPART_LOG) << "Current history: index=" << history->currentItemIndex() << "url=" << history->currentItem().url();
         QByteArray histData;
         QBuffer buff (&histData);
         m_historyData.clear();
@@ -806,7 +810,7 @@ void WebEngineBrowserExtension::saveHistory()
         QWidget* frameWidget = mainWidget ? mainWidget->parentWidget() : nullptr;
         if (frameWidget) {
             emit saveHistory(frameWidget, m_historyData);
-            // kDebug() << "# of items:" << history->count() << "current item:" << history->currentItemIndex() << "url:" << history->currentItem().url();
+            // qCDebug(WEBENGINEPART_LOG) << "# of items:" << history->count() << "current item:" << history->currentItemIndex() << "url:" << history->currentItem().url();
         }
     } else {
         Q_ASSERT(false); // should never happen!!!
@@ -815,14 +819,16 @@ void WebEngineBrowserExtension::saveHistory()
 
 void WebEngineBrowserExtension::slotPrintPreview()
 {
-#if 0
-    // Make it non-modal, in case a redirection deletes the part
-    QPointer<QPrintPreviewDialog> dlg (new QPrintPreviewDialog(view()));
-    connect(dlg.data(), SIGNAL(paintRequested(QPrinter*)),
-            view()->page()->currentFrame(), SLOT(print(QPrinter*)));
-    dlg->exec();
-    delete dlg;
-#endif
+    QPrinter printer;
+    QPrintPreviewDialog dlg(&printer, view());
+    auto printPreview = [this](QPrinter *p){
+        QEventLoop loop;
+        auto preview = [&](bool) {loop.quit();};
+        m_view->page()->print(p, preview);
+        loop.exec();
+    };
+    connect(&dlg, &QPrintPreviewDialog::paintRequested, this, printPreview);
+    dlg.exec();
 }
 
 void WebEngineBrowserExtension::slotOpenSelection()
@@ -837,9 +843,9 @@ void WebEngineBrowserExtension::slotOpenSelection()
 
 void WebEngineBrowserExtension::slotLinkInTop()
 {
-#if QTWEBENGINE_VERSION >= QT_VERSION_CHECK(5, 7, 0)
-    if (!view())
+    if (!view()) {
         return;
+    }
 
     KParts::OpenUrlArguments uargs;
     uargs.setActionRequestedByUser(true);
@@ -850,7 +856,6 @@ void WebEngineBrowserExtension::slotLinkInTop()
     const QUrl url(view()->contextMenuResult().linkUrl());
 
     emit openUrlRequest(url, uargs, bargs);
-#endif
 }
 
 ////
@@ -858,7 +863,7 @@ void WebEngineBrowserExtension::slotLinkInTop()
 WebEngineTextExtension::WebEngineTextExtension(WebEnginePart* part)
     : KParts::TextExtension(part)
 {
-    connect(part->view(), SIGNAL(selectionChanged()), this, SIGNAL(selectionChanged()));
+    connect(part->view(), &QWebEngineView::selectionChanged, this, &KParts::TextExtension::selectionChanged);
 }
 
 WebEnginePart* WebEngineTextExtension::part() const
@@ -940,7 +945,6 @@ static KParts::SelectorInterface::Element convertWebElement(const QWebElement& w
     }
     return element;
 }
-#endif
 
 static QString queryOne(const QString& query)
 {
@@ -987,6 +991,7 @@ static QList<KParts::SelectorInterface::Element> convertSelectionElements(const
     }
     return elements;
 }
+#endif
 
 KParts::SelectorInterface::Element WebEngineHtmlExtension::querySelector(const QString& query, KParts::SelectorInterface::QueryMethod method) const
 {
@@ -1130,7 +1135,7 @@ bool WebEngineHtmlExtension::setHtmlSettingsProperty(KParts::HtmlSettingsInterfa
             //settings->setAttribute(QWebEnngineSettings::PrivateBrowsingEnabled, value.toBool());
             return false;
         case KParts::HtmlSettingsInterface::UserDefinedStyleSheetURL:
-            //kDebug() << "Setting user style sheet for" << page << "to" << value.toUrl();
+            //qCDebug(WEBENGINEPART_LOG) << "Setting user style sheet for" << page << "to" << value.toUrl();
           //  settings->setUserStyleSheetUrl(value.toUrl());
             return false;
         default:
@@ -1163,7 +1168,7 @@ bool WebEngineScriptableExtension::setException (KParts::ScriptableExtension* ca
 
 QVariant WebEngineScriptableExtension::get (KParts::ScriptableExtension* callerPrincipal, quint64 objId, const QString& propName)
 {
-    //kDebug() << "caller:" << callerPrincipal << "id:" << objId << "propName:" << propName;
+    //qCDebug(WEBENGINEPART_LOG) << "caller:" << callerPrincipal << "id:" << objId << "propName:" << propName;
     return callerPrincipal->get (nullptr, objId, propName);
 }
 
@@ -1185,7 +1190,7 @@ QVariant WebEngineScriptableExtension::evaluateScript (KParts::ScriptableExtensi
 {
     Q_UNUSED(contextObjectId);
     Q_UNUSED(code)
-    //kDebug() << "principal:" << callerPrincipal << "id:" << contextObjectId << "language:" << lang << "code:" << code;
+    //qCDebug(WEBENGINEPART_LOG) << "principal:" << callerPrincipal << "id:" << contextObjectId << "language:" << lang << "code:" << code;
 
     if (lang != ECMAScript)
         return exception("unsupported language");
